#encoding=utf-8
#Данный пример считывает текущее состояние каналов модуля.
from pyiArduinoI2Crelay import *										# Подключаем библиотеку (модуль) для работы с реле
relay = pyiArduinoI2Crelay()                                         # Объявляем объект relay для работы с функциями и методами библиотеки iarduino_I2C_Relay, указывая адрес модуля на шине I2C.
																		# Если объявить объект без указания адреса (iarduino_I2C_Relay relay;), то адрес будет найден автоматически.
#	Включаем и выключаем каналы модуля: 								#
relay.digitalWrite(1, LOW);        	                             		# Отключаем 1 канал.
relay.digitalWrite(2,HIGH);         	                            	# Включаем  2 канал.
relay.digitalWrite(3, LOW);             	                        	# Отключаем 3 канал.
relay.digitalWrite(4,HIGH);                 	                    	# Включаем  4 канал.
#  Проверяем состояние каналов модуля в цикле:							#											
for i in range(4):														# Прозодим по всем каналам модуля.
	print("Канал № %s" % (i), end='')									# Выводим номер очередного канала.
	if relay.digitalRead(i):											# Если функция digitalRead() вернула True
		print(" включен ", end='')										# значит канал включён.
	else:																# Если функция digitalRead() вернула False 
		print(" отключен\t", end='')									# значит канал отключён.
print("--------")	
#  ПРИМЕЧАНИЕ: состояние всех каналов можно получить одним байтом: 
#  uint8_t j = relay.digitalRead(ALL_CHANNEL);                        
# 4 младших бита переменной «j» соответствуют состояниям 4 каналов модуля.																	
