// Данный пример демонстрирует программную перезагрузку модуля.     
#include <iostream>                                                 //	Подключаем библеотеку для работы с потоками входа/выхода
#include "iarduino_I2C_Relay.h"                                     //  Подключаем библиотеку для работы с реле и силовыми ключами.
iarduino_I2C_Relay relay(0x09);                                     //  Объявляем объект relay для работы с функциями и методами библиотеки iarduino_I2C_Relay, указывая адрес модуля на шине I2C.
                                                                    //  Если объявить объект без указания адреса (iarduino_I2C_Relay relay;), то адрес будет найден автоматически.
void loop(void);													//	Объявляем прототип функции loop()

int main(){                                                       	//	Главная функция программы
//Готовим модуль к работе:                                        	//
  relay.begin();                                                  	//  Инициируем работу с модулем.
	loop();															//	Входим в бесконечный цикл
}                                                                   //
                                                                    //
void loop(){                                                        //
	while(1){														// 	Бесконечный цикл
//  Включаем все каналы модуля:                                     //
    relay.digitalWrite(ALL_CHANNEL,HIGH);                           //  Включаем все каналы модуля.
    delay(2000);                                                    //  Ждём две секунды.
//  Перезагружаем модуль:                                           //
    relay.reset();                                                  //  После перезагрузки все регистры сбросятся в значения по умолчанию, все каналы будут отключены.
    delay(2000);                                                    //  Ждём две секунды.
	}
}                                                                   //
